package com.cultural.heritage.service.common.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.IService;
import com.cultural.heritage.service.common.CommonService;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import java.lang.reflect.Field;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

@Service
public class CommonServiceImpl implements CommonService {


    /**
     * 取出一个集合的某个id列表, 去另外一个集合中筛选出属于这个id列表的集合
     */
    @Override
    public <T, R> List<R> findByOtherIds(List<T> sourceList, IService<R> genericService, Function<T, Long> getId) {
        // 提取ID
        List<Long> ids = sourceList.stream()
                .map(getId)  // 提取每个元素的ID
                .collect(Collectors.toList());

        if (ids.isEmpty()) {
            return List.of();  // 返回空集合
        }
        // 构造查询条件
        QueryWrapper<R> queryWrapper = new QueryWrapper<>();
        queryWrapper.in("id", ids);

        // 返回查询结果
        return genericService.list(queryWrapper);
    }


    /**
     * 根据 ID 列表获取对应的实体列表
     */
    @Override
    public <T> List<T> findByMyIds(List<Long> ids, IService<T> genericService) {
        if (ids.isEmpty()) {
            return List.of();  // 返回空集合
        }
        // 构造查询条件
        QueryWrapper<T> queryWrapper = new QueryWrapper<>();
        queryWrapper.in("id", ids);  // 使用 IN 条件

        // 调用具体的服务执行查询
        return genericService.list(queryWrapper);
    }




    /**
     * 从第一个集合中提取某个属性值，并用这些值作为查询条件去查询第二个集合的数据。
     * @param sourceList 原始集合（源数据）
     * @param service 要执行查询的 Service
     * @param sourceField 在原始集合中提取的字段名
     * @param targetField 在目标集合中的查询字段名
     * @param <T> 目标查询实体类型
     * @return 查询结果集合
     */
    @Override
    public <T> List<T> findBySourceFieldValues(List<?> sourceList, IService<T> service, String sourceField, String targetField) {
        // 使用反射获取源集合中对应字段的值
        List<Object> fieldValues = sourceList.stream()
                .map(item -> getFieldValue(item, sourceField)) // 获取字段值
                .collect(Collectors.toList());

        // 如果 fieldValues 为空，直接返回空集合
        if (fieldValues.isEmpty()) {
            return List.of();  // 返回空集合
        }
        // 创建查询条件
        QueryWrapper<T> queryWrapper = new QueryWrapper<>();
        queryWrapper.in(targetField, fieldValues);  // 根据字段值进行查询

        return service.list(queryWrapper);  // 执行查询并返回结果
    }



    /**
     * 使用反射获取对象的字段值
     * @param object 对象
     * @param fieldName 字段名
     * @return 字段的值
     */
    private Object getFieldValue(Object object, String fieldName) {
        try {
            Field field = object.getClass().getDeclaredField(fieldName);
            field.setAccessible(true);
            return field.get(object);
        } catch (NoSuchFieldException | IllegalAccessException e) {
            throw new RuntimeException("字段获取失败", e);
        }
    }



    /**
     * 根据指定字段名和值，使用给定的服务对象查询对应的实体类列表。
     * @param <T> 实体类类型
     * @param fieldName 查询字段的名称
     * @param fieldValue 查询字段的值
     * @param service 用于执行查询的服务层对象
     *
     * @return 返回符合条件的实体类列表（`List<T>`）。如果没有符合条件的记录，返回空列表。
     */
    @Override
    public <T> List<T> findByFieldValue(String fieldName, Object fieldValue, IService<T> service) {
        // 创建 QueryWrapper，动态构建查询条件
        QueryWrapper<T> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(fieldName, fieldValue);  // 设置等值查询条件

        // 执行查询
        return service.list(queryWrapper);
    }




    /**
     * 根据多个字段和对应的值进行查询。
     * 该方法根据输入的查询条件，动态构建查询，并执行数据库查询。
     * @param fieldConditions 查询条件的字段和值，使用Map存储
     * @param service         执行查询操作的服务对象（通常是MyBatis-Plus的 IService）
     * @return               返回查询结果的列表
     */
    @Override
    public <T> List<T> findByMultipleFieldValues(Map<String, Object> fieldConditions, IService<T> service) {
        // 创建 QueryWrapper，动态构建查询条件
        QueryWrapper<T> queryWrapper = new QueryWrapper<>();

        // 遍历传入的条件Map，逐个设置查询条件
        for (Map.Entry<String, Object> entry : fieldConditions.entrySet()) {
            // 设置等值查询条件
            queryWrapper.eq(entry.getKey(), entry.getValue());
        }

        // 执行查询，并返回结果
        return service.list(queryWrapper);
    }




    /**
     * 将源列表 List<T> 转换为目标类型的列表 List<R>
     * @param sourceList 源列表，包含 T 类型的对象
     * @param targetClass 目标类型的 Class 对象
     * @param <T> 源类型
     * @param <R> 目标类型
     * @return 转换后的目标类型的 List
     */
    @Override
    public <T, R> List<R> convertList(List<T> sourceList, Class<R> targetClass) {
        // 使用 Stream 流式操作来遍历源列表，并将每个元素转换为目标类型
        return sourceList.stream()
                .map(source -> {
                    try {
                        // 通过反射创建目标类型的实例
                        R target = targetClass.getDeclaredConstructor().newInstance();
                        // 使用 BeanUtils.copyProperties 复制属性
                        BeanUtils.copyProperties(source, target);
                        return target;
                    } catch (Exception e) {
                        // 捕获异常并抛出运行时异常
                        throw new RuntimeException("Error copying properties", e);
                    }
                })
                .collect(Collectors.toList());  // 将转换后的对象收集到目标类型的列表中

    }


}
